﻿using Infragistics.Controls.Charts;
using Infragistics.Windows;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace XamDataChartDrawLineOnCanvasCase
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        List<double> doubles = new List<double>() { 4 };
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = new ViewModel();
        }

        #region Create Line
        Line line;

        private void chart_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            Canvas c = GetCanvas(chart.Series[0]);
            Point p = e.GetPosition(c);

            if (c != null)
            {
                line = new Line();

                line.StrokeThickness = 2;
                line.Stroke = Brushes.Blue;

                line.X1 = p.X;
                line.Y1 = p.Y;

                line.StrokeDashArray = new DoubleCollection(doubles);

                c.Children.Add(line);
            }
        }

        private void chart_MouseMove(object sender, MouseEventArgs e)
        {
            
            if (Mouse.LeftButton == MouseButtonState.Pressed && line.Tag == null)
            {
                Canvas c = GetCanvas(chart.Series[0]);

                Point p = e.GetPosition(c);

                if (c != null)
                {
                    line.X2 = p.X;
                    line.Y2 = p.Y;
                }
            }
        }

        private void chart_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (line.Tag == null)
            {
                Point startPoint = GetUnscaledValuesForMousePosition(line.X1, line.Y1);
                Point endPoint = GetUnscaledValuesForMousePosition(line.X2, line.Y2);
                line.Tag = new LineTagPoints() { StartPoint = startPoint, EndPoint = endPoint };
            }            
        }
        #endregion

        #region Rearrange Line After Zoom/Pan or Window Resize
        private void chart_WindowRectChanged(object sender, Infragistics.RectChangedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action(() =>
            {
                Canvas c = GetCanvas(chart.Series[0]);
                RearrangeLinePoints(c);
            }));
        }

        private void chart_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            Dispatcher.BeginInvoke(new Action(() =>
            {
                Canvas c = GetCanvas(chart.Series[0]);
                RearrangeLinePoints(c);             
            }));
        }
        #endregion

        #region Helper Methods
        
        public Canvas GetCanvas(Series series)
        {
            Canvas c = Utilities.GetDescendantFromName(series, "RootCanvas") as Canvas;
            return c;
        }

        public Point GetUnscaledValuesForMousePosition(double x, double y)
        {
            double xValue = xAxis.UnscaleValue(x);
            double yValue = yAxis.UnscaleValue(y);

            return new Point(xValue, yValue);
        }

        public void RearrangeLinePoints(Canvas c)
        {
            foreach (var x in c.Children)
            {
                if (x is Line)
                {
                    Line existingLine = x as Line;

                    if (existingLine.Tag is LineTagPoints)
                    {
                        LineTagPoints ltp = existingLine.Tag as LineTagPoints;

                        existingLine.X1 = xAxis.ScaleValue(ltp.StartPoint.X);
                        existingLine.Y1 = yAxis.ScaleValue(ltp.StartPoint.Y);

                        existingLine.X2 = xAxis.ScaleValue(ltp.EndPoint.X);
                        existingLine.Y2 = yAxis.ScaleValue(ltp.EndPoint.Y);
                    }
                }
            }
        }

        #endregion

     

    }
}
